# Script to generate SVGFlags.h
# Copyright 2003 Nikolas Zimmermann <wildfox@kde.org>

use POSIX;

my @properties;

my $curdir=`pwd`;
chomp $curdir;

my $splitcount;

# Open files
open(WRITE, ">" . $curdir . "/data/SVGFlags.h") || die($!);
open(READ, $curdir . "/data/SVGFlags.list") || die($!);

# Write header
print "Writing header...\n";
print WRITE << "EOF";
/*
    Copyright (C) 2003 KSVG Team
    This file is part of the KDE project

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef SVGFlags_H
#define SVGFlags_H

// Don't generate the SVG_*FLAGS*, manually keep in sync! (Niko)
#define SVG_FLAGS_1										reinterpret_cast<SVGObjectImpl *>(this)->flags1()
#define SVG_FLAGS_2										reinterpret_cast<SVGObjectImpl *>(this)->flags2()
#define SVG_STYLE_FLAGS									reinterpret_cast<SVGStylableImpl *>(this)->styleFlags()

#define SVG_FLAG_NONE									0x0000000000000000ULL
EOF

# Read properties
my $curflag;
while(<READ>) {
	$line=$_;
	chomp $line;

	if($line eq "") {
		next;
	}

	$find=index($line, "_FLAG:");

	if($find >= 0) {
		$curflag=substr($line, 0, length($line) - 1);
	} else {
		$prop=$line;

		$fmtprop=uc($prop);
		$fmtprop =~ s/-/_/g;
		
		push(@properties, $curflag . "|" . $fmtprop);
	}
}


# Sort properties
@properties=sort(@properties);

# Write flags, sorted
my $switch=1;
my $lastflag="";

$splitcount=1;

my $nullsafter, $nulls;

foreach(@properties) {
	$flag=substr($_, 0, index($_, "|"));
	$prop=substr($_, index($_, "|") + 1);

	if($nulls eq -1) {
		$nulls = 15;
		$nullsafter=0;
		$splitcount++;
	}

	$flag .= "_" . $splitcount;

	if($lastflag ne $flag) {
		$cmplast=substr($lastflag, 0, rindex($lastflag, "_"));
		$cmpcur=substr($flag, 0, rindex($flag, "_"));

		if($cmplast ne $cmpcur) {
			$splitcount=1;
			$flag = $cmpcur . "_" . $splitcount;

			print WRITE "\n";
		}

		$switch=1;
		$nullsafter=0;
		$nulls=15;
	}

	$string = "#define SVG_" . $flag . "_" . $prop;

	# Format output *hacky*
	if(length($string) <= 23) {
		$string .= "\t" x 9;
	} elsif(length($string) <= 27) {
		$string .= "\t" x 8;
	} elsif(length($string) > 27 and length($string) <= 31) {
		$string .= "\t" x 7;
	} elsif(length($string) > 31 and length($string) <= 35) {
		$string .= "\t" x 6;
	} elsif(length($string) > 35 and length($string) <= 39) {
		$string .= "\t" x 5;
	} elsif(length($string) > 39 and length($string) <= 43) {
		$string .= "\t" x 4;
	} elsif(length($string) > 43 and length($string) <= 48) {
		$string .= "\t" x 3;
	} elsif(length($string) > 48 and length($string) <= 51) {
		$string .= "\t" x 2;
	} else {
		$string .= "\t" x 1;
	}

	# Generate bitmask
	$string .= "0x" . "0" x $nulls;

	if($switch eq 1 || $switch eq 2) {
		$string .= $switch;
	} elsif($switch == 3) {
		$string .= "4";
	} elsif($switch == 4) {
		$string .= "8";
	}

	$string .= "0" x $nullsafter;
	$string .= "ULL";
	
	if($switch eq 4) {
		$nulls--;
		$nullsafter++;
		$switch=0;
	}

	print WRITE $string . "\n";

	$lastflag=$flag;
	$switch++;
}

# Finish file
print "Finishing file...\n";
print WRITE << "EOF";

#endif

// vim:ts=4:noet
EOF

close(READ) || warn($!);
close(WRITE) || warn($!);
